@@ -9,18 +9,17 @@
 #include "ProjectHistory.h"
 #include "ProjectRate.h"
 #include "ProjectSnap.h"
-#include "../ProjectSelectionManager.h"
-#include "../ProjectSettings.h"
-#include "../ProjectWindow.h"
 #include "../ProjectWindows.h"
 #include "../SelectUtilities.h"
 #include "SyncLock.h"
 #include "../TrackPanel.h"
+#include "Viewport.h"
 #include "WaveClip.h"
 #include "WaveTrack.h"
+#include "WaveTrackUtilities.h"
 #include "../LabelTrack.h"
-#include "../commands/CommandContext.h"
-#include "../commands/CommandManager.h"
+#include "CommandContext.h"
+#include "MenuRegistry.h"
 #include "../toolbars/ControlToolBar.h"
 #include "../tracks/ui/SelectHandle.h"
 #include "../tracks/labeltrack/ui/LabelTrackView.h"
@@ -216,18 +215,17 @@ void MoveWhenAudioInactive
    auto &tracks = TrackList::Get(project);
    auto &ruler = AdornedRulerPanel::Get(project);
    const auto &settings = ProjectSnap::Get(project);
-   auto &window = ProjectWindow::Get(project);
+   auto &viewport = Viewport::Get(project);
 
    // If TIME_UNIT_SECONDS, snap-to will be off.
    auto snapMode = settings.GetSnapMode();
-   const double t0 = viewInfo.selectedRegion.t0();
 
    // Move the cursor
    // Already in cursor mode?
-   if( viewInfo.selectedRegion.isPoint() )
+   if (viewInfo.selectedRegion.isPoint())
    {
-      double newT = OffsetTime(project,
-         t0, seekStep, timeUnit, snapMode);
+      double newT = OffsetTime(
+         project, viewInfo.selectedRegion.t0(), seekStep, timeUnit, snapMode);
       // constrain.
       newT = std::max(0.0, newT);
       // Move
@@ -243,15 +241,28 @@ void MoveWhenAudioInactive
    else
    {
       // Transition to cursor mode.
-      if( seekStep < 0 )
+      constexpr auto maySwapBoundaries = false;
+      if (seekStep < 0)
+      {
+         if (snapMode != SnapMode::SNAP_OFF)
+            viewInfo.selectedRegion.setT0(
+               settings.SnapTime(viewInfo.selectedRegion.t0()).time,
+               maySwapBoundaries);
          viewInfo.selectedRegion.collapseToT0();
+      }
       else
+      {
+         if (snapMode != SnapMode::SNAP_OFF)
+            viewInfo.selectedRegion.setT1(
+               settings.SnapTime(viewInfo.selectedRegion.t1()).time,
+               maySwapBoundaries);
          viewInfo.selectedRegion.collapseToT1();
+      }
       trackPanel.Refresh(false);
    }
 
    // Make sure NEW position is in view
-   window.ScrollIntoView(viewInfo.selectedRegion.t1());
+   viewport.ScrollIntoView(viewInfo.selectedRegion.t1());
    return;
 }
 
@@ -262,7 +273,7 @@ SelectionOperation operation)
    auto &viewInfo = ViewInfo::Get(project);
    auto &tracks = TrackList::Get(project);
    const auto &settings = ProjectSnap::Get(project);
-   auto &window = ProjectWindow::Get(project);
+   auto &viewport = Viewport::Get(project);
 
    if (operation == CURSOR_MOVE)
    {
@@ -296,7 +307,7 @@ SelectionOperation operation)
       viewInfo.selectedRegion.setT1( newT );
 
    // Ensure it is visible
-   window.ScrollIntoView(newT);
+   viewport.ScrollIntoView(newT);
 }
 
 // Handle small cursor and play head movements
@@ -358,7 +369,7 @@ void DoBoundaryMove(AudacityProject &project, int step, SeekInfo &info)
 {
    auto &viewInfo = ViewInfo::Get(project);
    auto &tracks = TrackList::Get(project);
-   auto &window = ProjectWindow::Get(project);
+   auto &viewport = Viewport::Get(project);
 
    // step is negative, then is moving left.  step positive, moving right.
    // Move the left/right selection boundary, to expand the selection
@@ -407,7 +418,7 @@ void DoBoundaryMove(AudacityProject &project, int step, SeekInfo &info)
       viewInfo.selectedRegion.setT1( newT );
 
    // Ensure it is visible
-   window.ScrollIntoView(newT);
+   viewport.ScrollIntoView(newT);
 
    ProjectHistory::Get( project ).ModifyState(false);
 }
@@ -441,7 +452,7 @@ void OnSelectAll(const CommandContext &context)
    //Presumably, there might be not more than one track
    //that expects text input
    for (auto wt : tracks.Any<WaveTrack>()) {
-      auto& view = WaveChannelView::Get(*wt);
+      auto& view = WaveChannelView::GetFirst(*wt);
       if (view.SelectAllText(context.project)) {
          trackPanel.Refresh(false);
          return;
@@ -474,7 +485,7 @@ void OnSelectSyncLockSel(const CommandContext &context)
 
    bool selected = false;
    for (auto t : tracks.Any() + &Track::SupportsBasicEditing
-         + &SyncLock::IsSyncLockSelected - &Track::IsSelected) {
+         + &SyncLock::IsSyncLockSelectedP - &Track::IsSelected) {
       t->SetSelected(true);
       selected = true;
    }
@@ -571,14 +582,14 @@ void OnSelectionRestore(const CommandContext &context)
 {
    auto &project = context.project;
    auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   auto &window = ProjectWindow::Get(project);
+   auto &viewport = Viewport::Get(project);
 
    if ((mRegionSave.t0() == 0.0) &&
        (mRegionSave.t1() == 0.0))
       return;
 
    selectedRegion = mRegionSave;
-   window.ScrollIntoView(selectedRegion.t0());
+   viewport.ScrollIntoView(selectedRegion.t0());
 
    ProjectHistory::Get( project ).ModifyState(false);
 }
@@ -631,21 +642,21 @@ void OnZeroCrossing(const CommandContext &context)
    // it if any stretched clip is involved.
    const auto projectRate = ProjectRate(project).GetRate();
    const auto searchWindowDuration = GetWindowSize(projectRate) / projectRate;
-   const auto wouldSearchStretchedClip =
+   const auto wouldSearchClipWithPitchOrSpeed =
       [searchWindowDuration](const WaveTrack& track, double t) {
-         const auto clips = track.GetClipsIntersecting(
+         const auto clips = WaveTrackUtilities::GetClipsIntersecting(track,
             t - searchWindowDuration / 2, t + searchWindowDuration / 2);
-         return std::any_of(
+         return any_of(
             clips.begin(), clips.end(),
-            [](const std::shared_ptr<const WaveClip>& clip) {
-               return !clip->StretchRatioEquals(1);
-            });
+            [](const auto& clip) { return clip->HasPitchOrSpeed(); });
       };
    const auto selected = tracks.Selected<const WaveTrack>();
    if (std::any_of(
           selected.begin(), selected.end(), [&](const WaveTrack* track) {
-             return wouldSearchStretchedClip(*track, selectedRegion.t0()) ||
-                    wouldSearchStretchedClip(*track, selectedRegion.t1());
+             return wouldSearchClipWithPitchOrSpeed(
+                       *track, selectedRegion.t0()) ||
+                    wouldSearchClipWithPitchOrSpeed(
+                       *track, selectedRegion.t1());
           }))
    {
       using namespace BasicUI;
@@ -689,16 +700,16 @@ void OnSnapToPrior(const CommandContext &context)
 void OnSelToStart(const CommandContext &context)
 {
    auto &project = context.project;
-   auto &window = ProjectWindow::Get( project );
-   window.Rewind(true);
+   auto &viewport = Viewport::Get(project);
+   viewport.ScrollToStart(true);
    ProjectHistory::Get( project ).ModifyState(false);
 }
 
 void OnSelToEnd(const CommandContext &context)
 {
    auto &project = context.project;
-   auto &window = ProjectWindow::Get( project );
-   window.SkipEnd(true);
+   auto &viewport = Viewport::Get(project);
+   viewport.ScrollToEnd(true);
    ProjectHistory::Get( project ).ModifyState(false);
 }
 
@@ -747,22 +758,22 @@ void OnCursorSelStart(const CommandContext &context)
 {
    auto &project = context.project;
    auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   auto &window = ProjectWindow::Get( project );
+   auto &viewport = Viewport::Get(project);
 
    selectedRegion.collapseToT0();
    ProjectHistory::Get( project ).ModifyState(false);
-   window.ScrollIntoView(selectedRegion.t0());
+   viewport.ScrollIntoView(selectedRegion.t0());
 }
 
 void OnCursorSelEnd(const CommandContext &context)
 {
    auto &project = context.project;
    auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   auto &window = ProjectWindow::Get( project );
+   auto &viewport = Viewport::Get(project);
 
    selectedRegion.collapseToT1();
    ProjectHistory::Get( project ).ModifyState(false);
-   window.ScrollIntoView(selectedRegion.t1());
+   viewport.ScrollIntoView(selectedRegion.t1());
 }
 
 void OnCursorTrackStart(const CommandContext &context)
@@ -770,7 +781,7 @@ void OnCursorTrackStart(const CommandContext &context)
    auto &project = context.project;
    auto &tracks = TrackList::Get(project);
    auto &selectedRegion = ViewInfo::Get(project).selectedRegion;
-   auto &window = ProjectWindow::Get(project);
+   auto &viewport = Viewport::Get(project);
 
    double kWayOverToRight = std::numeric_limits<double>::max();
 
@@ -788,7 +799,7 @@ void OnCursorTrackStart(const CommandContext &context)
 
    selectedRegion.setTimes(minOffset, minOffset);
    ProjectHistory::Get(project).ModifyState(false);
-   window.ScrollIntoView(selectedRegion.t0());
+   viewport.ScrollIntoView(selectedRegion.t0());
 }
 
 void OnCursorTrackEnd(const CommandContext &context)
@@ -796,7 +807,7 @@ void OnCursorTrackEnd(const CommandContext &context)
    auto &project = context.project;
    auto &tracks = TrackList::Get(project);
    auto &selectedRegion = ViewInfo::Get(project).selectedRegion;
-   auto &window = ProjectWindow::Get(project);
+   auto &viewport = Viewport::Get(project);
 
    double kWayOverToLeft = std::numeric_limits<double>::lowest();
 
@@ -814,7 +825,7 @@ void OnCursorTrackEnd(const CommandContext &context)
 
    selectedRegion.setTimes(maxEndOffset, maxEndOffset);
    ProjectHistory::Get(project).ModifyState(false);
-   window.ScrollIntoView(selectedRegion.t1());
+   viewport.ScrollIntoView(selectedRegion.t1());
 }
 
 void OnSkipStart(const CommandContext &context)
@@ -939,11 +950,10 @@ static CommandHandlerObject &findCommandHandler(AudacityProject &project) {
 #define FN(X) (& SelectActions::Handler :: X)
 
 namespace {
-using namespace MenuTable;
-BaseItemSharedPtr SelectMenu()
+using namespace MenuRegistry;
+auto SelectMenu()
 {
-   using Options = CommandManager::Options;
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
    ( FinderScope{ findCommandHandler },
    /* i18n-hint: (verb) It's an item on a menu. */
    Menu( wxT("Select"), XXO("&Select"),
@@ -1029,15 +1039,11 @@ BaseItemSharedPtr SelectMenu()
    return menu;
 }
 
-AttachedItem sAttachment1{
-   wxT(""),
-   Indirect(SelectMenu())
-};
+AttachedItem sAttachment1{ Indirect(SelectMenu()) };
 
-BaseItemSharedPtr ExtraSelectionMenu()
+auto ExtraSelectionMenu()
 {
-   using Options = CommandManager::Options;
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
    ( FinderScope{ findCommandHandler },
    Menu( wxT("Select"), XXO("&Selection"),
       Command( wxT("SnapToOff"), XXO("Snap-To &Off"), FN(OnSnapToOff),
@@ -1076,16 +1082,14 @@ BaseItemSharedPtr ExtraSelectionMenu()
    return menu;
 }
 
-AttachedItem sAttachment2{
-   wxT("Optional/Extra/Part1"),
-   Indirect(ExtraSelectionMenu())
+AttachedItem sAttachment2{ Indirect(ExtraSelectionMenu()),
+   wxT("Optional/Extra/Part1")
 };
 }
 
 namespace {
-BaseItemSharedPtr CursorMenu()
+auto CursorMenu()
 {
-   using Options = CommandManager::Options;
    static const auto CanStopFlags = AudioIONotBusyFlag() | CanStopAudioStreamFlag();
 
    // JKC: ANSWER-ME: How is 'cursor to' different to 'Skip To' and how is it
@@ -1093,7 +1097,7 @@ BaseItemSharedPtr CursorMenu()
    // GA: 'Skip to' moves the viewpoint to center of the track and preserves the
    // selection. 'Cursor to' does neither. 'Center at' might describe it better
    // than 'Skip'.
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
    ( FinderScope{ findCommandHandler },
    Menu( wxT("Cursor"), XXO("&Cursor to"),
       Command( wxT("CursSelStart"), XXO("Selection Star&t"),
@@ -1125,15 +1129,13 @@ BaseItemSharedPtr CursorMenu()
    return menu;
 }
 
-AttachedItem sAttachment0{
-   wxT("Transport/Basic"),
-   Indirect(CursorMenu())
+AttachedItem sAttachment0{ Indirect(CursorMenu()),
+   wxT("Transport/Basic")
 };
 
-BaseItemSharedPtr ExtraCursorMenu()
+auto ExtraCursorMenu()
 {
-   using Options = CommandManager::Options;
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
    ( FinderScope{ findCommandHandler },
    Menu( wxT("Cursor"), XXO("&Cursor"),
       Command( wxT("CursorLeft"), XXO("Cursor &Left"), FN(OnCursorLeft),
@@ -1158,15 +1160,13 @@ BaseItemSharedPtr ExtraCursorMenu()
    return menu;
 }
 
-AttachedItem sAttachment4{
-   wxT("Optional/Extra/Part2"),
-   Indirect(ExtraCursorMenu())
+AttachedItem sAttachment4{ Indirect(ExtraCursorMenu()),
+   wxT("Optional/Extra/Part2")
 };
 
-BaseItemSharedPtr ExtraSeekMenu()
+auto ExtraSeekMenu()
 {
-   using Options = CommandManager::Options;
-   static BaseItemSharedPtr menu{
+   static auto menu = std::shared_ptr{
    ( FinderScope{ findCommandHandler },
    Menu( wxT("Seek"), XXO("See&k"),
       Command( wxT("SeekLeftShort"), XXO("Short Seek &Left During Playback"),
@@ -1186,9 +1186,8 @@ BaseItemSharedPtr ExtraSeekMenu()
    return menu;
 }
 
-AttachedItem sAttachment5{
-   wxT("Optional/Extra/Part1"),
-   Indirect(ExtraSeekMenu())
+AttachedItem sAttachment5{ Indirect(ExtraSeekMenu()),
+   wxT("Optional/Extra/Part1")
 };
 
 }
